"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.killAll = exports.addProcess = exports.wasKillAll = void 0;
const ps_tree_1 = __importDefault(require("ps-tree"));
const child_process_1 = require("child_process");
const run_state_1 = require("./run-state");
const log_1 = require("./log");
const contracts_1 = require("./contracts");
let _wasKillAll;
function wasKillAll() {
    return _wasKillAll;
}
exports.wasKillAll = wasKillAll;
const processList = [];
function addProcess(proc) {
    processList.push(proc);
}
exports.addProcess = addProcess;
process.on('SIGTERM', () => {
    console.log('SIGTERM');
    killAll({ isFailure: true, syncKill: true });
});
process.on('SIGHUP', () => {
    console.log('SIGHUP');
    killAll({ isFailure: true, syncKill: true });
});
process.on('SIGINT', () => {
    console.log('SIGINT');
    killAll({ isFailure: true, syncKill: true });
});
process.on('SIGBREAK', () => {
    console.log('SIGBREAK');
    killAll({ isFailure: true, syncKill: true });
});
process.on('beforeExit', () => {
    // console.log('beforeExit')
    killAll({ isFailure: false, syncKill: false });
});
process.on('exit', () => {
    console.log('exit');
    killAll({ isFailure: false, syncKill: true });
});
// process.on('disconnect', killAll)
process.on('uncaughtException', err => {
    log_1.printError('uncaughtException', err);
    killAll({ isFailure: true, syncKill: false });
});
function _killByPidsUnix(...pids) {
    if (!pids.length) {
        return;
    }
    const params = pids.map(o => o.toString());
    params.unshift('-15');
    console.log(`kill ${params.join(' ')}`);
    child_process_1.spawn('kill', params, {
        detached: true,
        stdio: 'ignore',
    })
        // .on('error', err => printError('kill error', err))
        .unref();
}
function killByPidsUnix(...pids) {
    if (!pids.length) {
        return;
    }
    _killByPidsUnix(...pids);
    for (let i = 0; i < pids.length; i++) {
        ps_tree_1.default(pids[i], (err, children) => {
            if (err) {
                log_1.printError('psTree error', err);
                children = [];
            }
            _killByPidsUnix(...children.map(o => o.PID));
        });
    }
}
function killByPidsWindows(...pids) {
    if (!pids.length) {
        return;
    }
    const params = ['/F', '/T'];
    for (let i = 0; i < pids.length; i++) {
        params.push('/PID');
        params.push(pids[i].toString());
    }
    console.log(`taskkill ${params.join(' ')}`);
    child_process_1.spawn('taskkill', params, {
        detached: true,
        stdio: 'ignore',
    })
        // .on('error', err => printError('kill error', err))
        .unref();
}
function killByPids(...pids) {
    if (pids.length) {
        // console.log(`Kill All: ${pids.join(' ')}`)
        if (process.platform === 'win32') {
            killByPidsWindows(...pids);
        }
        else {
            killByPidsUnix(...pids);
        }
    }
}
function killAll({ isFailure, syncKill, }) {
    if (_wasKillAll) {
        return;
    }
    _wasKillAll = true;
    console.log('Terminating...');
    const kill = () => {
        const procs = processList.filter(o => o.pid && !o.killed && o.pid !== process.pid);
        const pids = procs.map(o => o.pid);
        log_1.printRunStates();
        killByPids(...pids);
        if (run_state_1.getRunStates().some(o => o.status === contracts_1.RunStatus.ERROR)) {
            isFailure = true;
        }
        run_state_1.clearRunStates();
        if (isFailure) {
            // eslint-disable-next-line no-process-exit
            process.exit(1);
        }
    };
    if (syncKill) {
        kill();
    }
    else {
        setTimeout(kill, 100);
    }
}
exports.killAll = killAll;
